'Code compilation originally from jconwell http://www.codeproject.com/dotnet/DotNetScript.asp

'Wrapper around compilation originally by dstang2000 (http://www.codeproject.com/dotnet/DynamicCompileAndRun.asp)

'help with writing a VS Add-in: Scott Swigart (http://msdn.microsoft.com/msdnmag/issues/06/02/PasteAs/)

'help with getting selected text: from Kevin McFarlane (http://www.codeproject.com/macro/vscsharpmacros.asp)


Imports System.Reflection.Assembly
Imports System.Reflection
Imports System.Diagnostics
Imports System.Threading
Imports System.CodeDom.Compiler
Imports System.Text
Imports Microsoft.CSharp

Public Class CompileEngine
  Private m_errors() As CompilerError

  Public Enum languagetype
    vb
    csharp
  End Enum

    Private m_SourceCode As String
    Private m_Exception As Exception = Nothing
    Public ReadOnly Property Exception() As Exception
        Get
            Return m_Exception
        End Get
    End Property
    Public Property sourceCode() As String
        Get
            Return m_SourceCode
        End Get
        Set(ByVal Value As String)
            m_SourceCode = Value
        End Set
    End Property

    Private m_EntryPoint As String
    Public Property EntryPoint() As String
        Get
            Return m_EntryPoint
        End Get
        Set(ByVal Value As String)
            m_EntryPoint = Value
        End Set
    End Property

    Private m_Language As String
    Public Property Language() As String
        Get
            Return m_Language
        End Get
        Set(ByVal Value As String)
            m_Language = Value
        End Set
    End Property


    Public Sub New(ByVal code As String, ByVal language As languagetype, ByVal entry As String)
        Me.sourceCode = code
        Me.Language = language
        Me.EntryPoint = entry

    End Sub

    Public Function Run() As CompilerError()

        ClearErrMsgs()

        Dim strRealSourceCode As String = PrepareRealSourceCode()

        Dim _assembly As System.Reflection.Assembly = createassembly(strRealSourceCode)

        CallEntry(_assembly, EntryPoint)

        Return m_errors

    End Function

    Private Sub ClearErrMsgs()
        m_errors = Nothing

    End Sub


    Private Function GetErrorMsg() As String
        Dim c As CompilerError
        Dim sb As New StringBuilder
        If Not m_errors Is Nothing Then
            For Each c In m_errors

                sb.AppendFormat("{0}({1}): {2} {3}{4}", c.Line, c.Column, c.ErrorNumber, c.ErrorText, vbCrLf)

            Next
        End If
        Return sb.ToString()
    End Function
    Private Function createassembly(ByVal strRealSourceCode As String) As System.Reflection.Assembly

        'create an instance of whichever code provider is needed
        Dim codeProvider As CodeDomProvider

        If (Language = languagetype.csharp) Then
            codeProvider = New CSharpCodeProvider
        ElseIf Language = languagetype.vb Then
            codeProvider = New VBCodeProvider
        End If

        'and create a language specific code compiler
        Dim compiler As ICodeCompiler = codeProvider.CreateCompiler()

        'add compiler parameters
        Dim compilerParams As New CompilerParameters

        compilerParams.CompilerOptions = "/target:library"

        'you can also add /optimize
        compilerParams.GenerateExecutable = False
        compilerParams.GenerateInMemory = True
        compilerParams.IncludeDebugInformation = False

        'add some basic references
        'okay -- this is a little dodgy, read 'magic'
        compilerParams.ReferencedAssemblies.Add("mscorlib.dll")
        compilerParams.ReferencedAssemblies.Add("System.dll")
        compilerParams.ReferencedAssemblies.Add("System.Data.dll")
        compilerParams.ReferencedAssemblies.Add("System.Drawing.dll")
        compilerParams.ReferencedAssemblies.Add("System.Xml.dll")
        compilerParams.ReferencedAssemblies.Add("System.Windows.Forms.dll")

        If (Language = languagetype.vb) Then
            compilerParams.ReferencedAssemblies.Add("Microsoft.VisualBasic.dll")
        End If

        'actually compile the code
        Dim results As CompilerResults = compiler.CompileAssemblyFromSource( _
                compilerParams, _
                strRealSourceCode)
        If results.Errors.Count = 0 Then
            'get a hold of the actual assembly that was generated
            'Dim generatedAssembly As System.Reflection.Assembly = 
            Return results.CompiledAssembly
        Else
            Dim c As CompilerError
            ReDim m_errors(results.Errors.Count - 1)
            Dim i As Integer = 0
            For Each c In results.Errors
                m_errors(i) = (c)
                i += 1
            Next
            Return Nothing
        End If

    End Function

    Public Function PrepareRealSourceCode() As String

        Dim strRealSourceCode As String = ""
        strRealSourceCode = sourceCode

        'for VB Prog, Add Main(), So We Can Run It
        If Language = languagetype.vb Then

            If EntryPoint = "Main" AndAlso _
             strRealSourceCode.IndexOf("Sub Main(") < 0 Then

                'Wrap it in a class... with a main.
                Dim posClass As Integer = strRealSourceCode.IndexOf("Public Class") + _
                   "Public Class ".Length
                Dim posClassEnd As Integer = strRealSourceCode.IndexOf(vbCrLf, Math.Min(strRealSourceCode.Length, posClass))
                If posClass > ("Public Class ".Length - 1) Then
                    Dim className As String = strRealSourceCode.Substring(posClass, _
                         posClassEnd - posClass)
                    Dim pos As Integer = strRealSourceCode.LastIndexOf("End Class")

                    If (pos > 0) Then
                        strRealSourceCode = strRealSourceCode.Substring(0, pos) & _
                             "Private Shared Sub Main()" & vbCrLf & _
                             "Dim frm As New " & className & "()" & vbCrLf & _
                             "End Sub" & vbCrLf & strRealSourceCode.Substring(pos)
                    End If
                    ' "If TypeOf frm Is Form Then frm.ShowDialog()" & vbCrLf & _
                Else
                    'No class declaration
                    'wrap the whole thing in class...
                    strRealSourceCode = _
                        "Public Class Fred" & vbCrLf & _
                        "  Public Sub New()" & vbCrLf & _
                        strRealSourceCode & vbCrLf & _
                        "  End Sub" & vbCrLf & _
                        "  Private Shared Sub Main()" & vbCrLf & _
                        "    Dim f As New Fred()" & vbCrLf & _
                        "  End Sub" & vbCrLf & _
                        "End Class"
                End If
            End If
            'Add some using (i.e. 'Imports') statements
        End If


        'HACK: this chunk of logic and the above one (for vb) are almost identical!!
        'it's just crying out to be refactored....
        If Language = languagetype.csharp Then
            If EntryPoint = "Main" AndAlso _
                         strRealSourceCode.IndexOf("void Main(") < 0 Then


                'Wrap it in a class... with a main.
                Dim posClass As Integer = strRealSourceCode.IndexOf("public class") + _
                   "public class ".Length
                Dim posClassEnd As Integer = strRealSourceCode.IndexOf(vbCrLf, Math.Min(strRealSourceCode.Length, posClass))
                If posClass > ("public class ".Length - 1) Then
                    Dim className As String = strRealSourceCode.Substring(posClass, _
                         posClassEnd - posClass)
                    Dim pos As Integer = strRealSourceCode.LastIndexOf("}") 'Dodgy!
                    'I assume that the last '}' is an endclass marker...
                    'it could easily be an end namespace, for example...
                    'but 

                    If (pos > 0) Then
                        strRealSourceCode = strRealSourceCode.Substring(0, pos) & _
                             "static void Main() {" & vbCrLf & _
                             className & " fred = new " & className & "();" & vbCrLf & _
                             "}" & vbCrLf & strRealSourceCode.Substring(pos)
                    End If
                    ' "If TypeOf frm Is Form Then frm.ShowDialog()" & vbCrLf & _
                Else
                    'No class declaration...
                    'so wrap the whole thing in a Fred class...

                    strRealSourceCode = _
                        "public class Fred {" & vbCrLf & _
                        "  public Fred() {" & vbCrLf & _
                        strRealSourceCode & vbCrLf & _
                        "  }" & vbCrLf & _
                        "  static void Main() {" & vbCrLf & _
                        "    Fred f = new Fred();" & vbCrLf & _
                        "  }" & vbCrLf & _
                        "}"
                End If

            End If
        End If


        strRealSourceCode = ImportStatements(strRealSourceCode, Language) + strRealSourceCode

        m_SourceCode = strRealSourceCode
        Return strRealSourceCode

    End Function

    Private Function ImportStatements(ByVal strRealSourceCode As String, ByVal Language As languagetype) As String
        Dim strUsingStatement As String = ""
        Dim importsStatement() As String
        Dim basicImportsStatement() As String = { _
         "Imports Microsoft.VisualBasic", _
         "Imports System", _
         "Imports System.Windows.Forms", _
         "Imports System.Drawing", _
         "Imports System.Data", _
         "Imports System.Threading", _
         "Imports System.Xml", _
         "Imports System.Collections", _
         "Imports System.Diagnostics"}

        Dim cSharpImportsStatement() As String = { _
             "using System;", _
             "using System.Windows.Forms;", _
             "using System.Drawing;", _
             "using System.Data;", _
             "using System.Threading;", _
             "using System.Xml;", _
             "using System.Collections;", _
             "using System.Diagnostics;"}


        Dim si As String

        If Language = languagetype.vb Then
            importsStatement = basicImportsStatement
        Else
            importsStatement = cSharpImportsStatement
        End If

        For Each si In importsStatement

            If (strRealSourceCode.IndexOf(si) < 0) Then
                strUsingStatement &= si & vbCrLf

            End If
        Next
        Return strUsingStatement.ToString()
    End Function


    Public Sub CallEntry(ByVal asmbly As System.Reflection.Assembly, ByVal entrypoint As String)
        Try

            'uses reflection to call the static Main function
            Dim mods As System.Reflection.Module() = asmbly.GetModules(False)

            Dim types() As Type = mods(0).GetTypes()

            Dim typ As Type
            For Each typ In types

                Dim mi As System.Reflection.MethodInfo = typ.GetMethod(entrypoint, _
                            BindingFlags.Public Or BindingFlags.NonPublic _
                                Or BindingFlags.Static)

                If Not mi Is Nothing Then

                    If (mi.GetParameters().Length = 1) Then

                        If (mi.GetParameters()(0).ParameterType.IsArray) Then
                            Dim par(0) As String
                            'if main has arguments... pass em in
                            mi.Invoke(Nothing, par)
                        End If

                    Else

                        mi.Invoke(Nothing, Nothing)
                    End If
                    Return

                End If

            Next
            m_Exception = Nothing
        Catch ex As Exception
            If Not ex.InnerException Is Nothing Then
                m_Exception = ex.InnerException
            Else
                m_Exception = ex
            End If

        End Try
    End Sub
End Class